'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactClass = require('react-class');

var _reactClass2 = _interopRequireDefault(_reactClass);

var _objectAssign = require('object-assign');

var _objectAssign2 = _interopRequireDefault(_objectAssign);

var _raf = require('raf');

var _raf2 = _interopRequireDefault(_raf);

var _reactFlex = require('react-flex');

var _shallowequal = require('shallowequal');

var _shallowequal2 = _interopRequireDefault(_shallowequal);

var _lodash = require('lodash.flatten');

var _lodash2 = _interopRequireDefault(_lodash);

var _getDataRangeToRender = require('./getDataRangeToRender');

var _getDataRangeToRender2 = _interopRequireDefault(_getDataRangeToRender);

var _join = require('../join');

var _join2 = _interopRequireDefault(_join);

var _getIndexBy = require('../utils/getIndexBy');

var _getIndexBy2 = _interopRequireDefault(_getIndexBy);

var _Column = require('../Column');

var _Column2 = _interopRequireDefault(_Column);

var _EmptyText = require('./EmptyText');

var _EmptyText2 = _interopRequireDefault(_EmptyText);

var _Scroller = require('./Scroller');

var _Scroller2 = _interopRequireDefault(_Scroller);

var _ColumnGroup = require('./ColumnGroup');

var _ColumnGroup2 = _interopRequireDefault(_ColumnGroup);

var _reactNotifyResize = require('react-notify-resize');

var _reactNotifyResize2 = _interopRequireDefault(_reactNotifyResize);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 *   Body is a statefull component, it keeps track of:
 *
 * - bodyHeight: it is the height of the this component, it is used
 *               to set the scroller height, and to calculate how many rows
 *               to be rendered, it is also used to determine when we hit scrollBottom
 *               to update we use `react-notify-resize`, to listen to size changes
 * - scrollTop
 * - overRowId: id of the row that has :hover
 * - maxScrollTop: is calculated based on contentHeight
 *                 (height if all rows would be rendered) + visible area height
 *                 this is updated when data is changed (no of rows changes) or
 *                 bodyHeight changes
 * - isScrolling
 * - isPlaceholderActive: - flag if row placeholder should be rendered
 *                        - it is set after 300ms(by default, controlled with `rowPlaceholderDelay` prop)
 *                          if the `isScrolling` flag is still true
 */

var Body = function (_Component) {
  _inherits(Body, _Component);

  function Body(props) {
    _classCallCheck(this, Body);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Body).call(this, props));

    var columns = _this.getNewColumns(props);
    var flatColumns = (0, _lodash2.default)(columns);

    _this.state = {
      columns: columns,
      flatColumns: flatColumns,
      bodyHeight: 0,
      scrollTop: props.defaultScrollTop,
      overRowId: null,
      maxScrollTop: props.defaultScrollTop,
      isScrolling: false,
      isPlaceholderActive: false
    };
    return _this;
  }

  _createClass(Body, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _this2 = this;

      this.setBodyHeight();

      setTimeout(function () {
        _this2.setMaxScrollTop();
      }, 0);
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps, nextState) {
      if (nextProps.data === null) {
        this.setState({
          scrollTop: 0
        });
      }

      if (nextProps.contentHeight !== this.props.contentHeight) {
        var newMaxScrollTop = this.setMaxScrollTop(nextProps.contentHeight, nextProps);

        if (newMaxScrollTop < 0) {
          this.scrollAt(0);
        } else if (this.state.scrollTop > newMaxScrollTop) {
          this.scrollAt(newMaxScrollTop);
        }
      }

      // we have to determine if any of the folowig has changed
      // - columns
      // - columngrups has changed children or column prop
      var newColumns = this.getNewColumns(nextProps);
      if (!(0, _shallowequal2.default)(newColumns, this.state.columns)) {
        var columns = this.getNewColumns(nextProps);
        var flatColumns = (0, _lodash2.default)(columns);

        this.setState({
          columns: columns,
          flatColumns: flatColumns
        });
      }
    }
  }, {
    key: 'componentWillUpdate',
    value: function componentWillUpdate(nextProps, nextState) {
      if (!this.refs.scroller) {
        return;
      }

      if (
      // if controlled set each time, so the scrollbar is forced to not move
      nextProps.scrollTop != null) {
        this.refs.scroller.scrollAt(nextProps.scrollTop);
      } else if (nextState.scrollTop !== this.state.scrollTop) {
        this.refs.scroller.scrollAt(nextState.scrollTop);
      }
    }

    // todo func getBodyHeight

  }, {
    key: 'render',
    value: function render() {
      var preparedProps = this.p = this.prepareProps(this.props);

      var data = preparedProps.data;
      var columns = preparedProps.columns;
      var loading = preparedProps.loading;
      var scrollTop = preparedProps.scrollTop;
      var resizeTool = preparedProps.resizeTool;


      var className = (0, _join2.default)('react-datagrid__body', this.state.isScrolling && 'react-datagrid__body--scrolling');

      return _react2.default.createElement(
        _reactFlex.Item,
        {
          flex: true,
          column: true,
          className: className,
          data: null,
          ref: 'body'
        },
        _react2.default.createElement('div', { className: 'react-datagrid__column-group__header react-datagrid__column-group__header--placeholder', style: { height: this.state.headerHeight, width: '100%' } }),
        resizeTool,
        this.renderScroller()
      );
    }
  }, {
    key: 'renderScroller',
    value: function renderScroller() {
      if (!this.props.data) {
        return;
      }

      return _react2.default.createElement(
        _Scroller2.default,
        {
          ref: 'scroller',
          contentHeight: this.p.contentHeight,
          headerHeight: this.p.headerHeight,
          onScroll: this.onScroll,
          scrollTop: this.p.scrollTop,
          height: this.state.bodyHeight,
          scrollbarWidth: this.props.scrollbarWidth,
          toggleIsScrolling: this.toggleIsScrolling,
          maxScrollTop: this.p.maxScrollTop,
          hasScroll: this.p.hasScroll
        },
        this.renderColumnGroups()
      );
    }
  }, {
    key: 'renderColumnGroups',
    value: function renderColumnGroups() {
      var preparedProps = this.p;
      var data = preparedProps.data;
      var columns = preparedProps.columns;
      var rowHeight = preparedProps.rowHeight;
      var contentHeight = preparedProps.contentHeight;
      var renderRow = preparedProps.renderRow;
      var rowStyle = preparedProps.rowStyle;
      var rowProps = preparedProps.rowProps;
      var selected = preparedProps.selected;
      var isMultiselect = preparedProps.isMultiselect;
      var hasSelection = preparedProps.hasSelection;
      var onRowClick = preparedProps.onRowClick;
      var rowFactory = preparedProps.rowFactory;
      var cellFactory = preparedProps.cellFactory;
      var extraRows = preparedProps.extraRows;
      var onColumnGroupScroll = preparedProps.onColumnGroupScroll;
      var activeIndex = preparedProps.activeIndex;
      var scrollTop = preparedProps.scrollTop;
      var children = preparedProps.children;
      var buffer = preparedProps.buffer;
      var zebraRows = preparedProps.zebraRows;
      var renderRowPlaceholder = preparedProps.renderRowPlaceholder;
      var rowKey = preparedProps.rowKey;
      var header = preparedProps.header;
      var onHeaderCellClick = preparedProps.onHeaderCellClick;
      var onHeaderSortClick = preparedProps.onHeaderSortClick;
      var isMultiSort = preparedProps.isMultiSort;
      var sortable = preparedProps.sortable;
      var sortInfo = preparedProps.sortInfo;


      var bodyHeight = this.p.bodyHeight;
      var bufferValid = true;

      /**
       * we need to buffer rendering
       * only rerender rows when buffer (half of extra rows height) is scrolled
       * and we need to render anoter set of rows
       * cache scrollTop and fromTo
       * const {from, to} = getDataRangeToRender(bodyHeight, rowHeight, scrollTop, extraRows)
      **/

      this.fromTo = (0, _getDataRangeToRender2.default)(bodyHeight, rowHeight, scrollTop, extraRows);

      if (Math.abs(this.oldScrollTop - scrollTop - rowHeight) >= buffer || !this.fromTo) {
        this.oldScrollTop = scrollTop;
        bufferValid = false;
      }

      var _fromTo = this.fromTo;
      var from = _fromTo.from;
      var to = _fromTo.to;

      var offsetTop = from * rowHeight;
      var innerWrapperOffset = offsetTop - scrollTop;

      // only send to columngroup the prop that it needs to know about
      var columnGroupProps = {
        data: data,
        rowHeight: rowHeight,
        isMultiselect: isMultiselect,
        hasSelection: hasSelection,
        rowProps: rowProps,
        from: from,
        to: to,
        renderRow: renderRow,
        rowStyle: rowStyle,
        rowFactory: rowFactory,
        cellFactory: cellFactory,
        selected: selected,
        activeIndex: activeIndex,
        innerWrapperOffset: innerWrapperOffset,
        zebraRows: zebraRows,
        bufferValid: bufferValid,
        renderRowPlaceholder: renderRowPlaceholder,
        rowKey: rowKey,
        header: header,
        onHeaderCellClick: onHeaderCellClick,
        onHeaderSortClick: onHeaderSortClick,
        sortable: sortable,
        sortInfo: sortInfo,
        isMultiSort: isMultiSort,
        onHeaderHeightChange: this.onHeaderHeightChange,
        isPlaceholderActive: this.state.isPlaceholderActive,
        isScrolling: this.state.isScrolling,
        viewportHeight: bodyHeight,
        globalProps: this.props,
        onRowMouseEnter: this.onRowMouseEnter,
        onRowMouseLeave: this.onRowMouseLeave,
        onRowClick: onRowClick,
        overRowId: this.state.overRowId,
        onScroll: onColumnGroupScroll
      };

      /**
       * If no coumnGroup is specified, create a ColumGroup with all passed columns
       */
      if (!children) {
        return _react2.default.createElement(_ColumnGroup2.default, _extends({}, columnGroupProps, {
          columns: columns,
          width: '100%'
        }));
      } else {
        /**
         * Children are specified, take each Columngroup and insert props
         */
        return _react2.default.Children.map(children, function (child, index) {
          return _react2.default.cloneElement(child, (0, _objectAssign2.default)({}, columnGroupProps, child.props, // let columngroup props overwrite those passed
          { columns: columns[index] }));
        });
      }
    }
  }, {
    key: 'onHeaderHeightChange',
    value: function onHeaderHeightChange(height) {
      var _this3 = this;

      this.setBodyHeight(height);
      this.setState({
        headerHeight: height
      });
      setTimeout(function () {
        _this3.setMaxScrollTop();
      }, 0);
    }
  }, {
    key: 'onRowMouseEnter',
    value: function onRowMouseEnter(event, rowProps) {
      this.setState({
        overRowId: rowProps.id
      });

      this.p.onRowMouseEnter(event, rowProps);
    }
  }, {
    key: 'onRowMouseLeave',
    value: function onRowMouseLeave(event, rowProps) {
      // remove id if still present
      if (this.state.overRowId === rowProps.id) {
        this.setState({
          overRowId: null
        });
      }

      this.p.onRowMouseLeave(event, rowProps);
    }
  }, {
    key: 'onScroll',
    value: function onScroll(scrollTop, event) {
      var _this4 = this;

      this.scrollAt(scrollTop);

      if (this.p.onScroll) {
        this.p.onScroll(scrollTop, event);
      }

      this.toggleIsScrolling();

      if (!this.state.isScrolling) {
        // at this ponint scrolling starts
        this.setState({
          isScrolling: true
        });

        // if it is still scrolling after `rowPlaceholderDelay`ms (defaults to 300ms)
        // set `isPlaceholderActive` to true, to announce that row placeholder can be
        // rendered
        if (this.props.rowPlaceholder) {
          setTimeout(function () {
            if (_this4.state.isScrolling) {
              _this4.setState({
                isPlaceholderActive: true
              });
            }
          }, this.props.rowPlaceholderDelay);
        }
      }
    }
  }, {
    key: 'onResize',
    value: function onResize() {
      var _this5 = this;

      this.setBodyHeight();
      setTimeout(function () {
        _this5.setMaxScrollTop();
      }, 0);
    }
  }, {
    key: 'setBodyHeight',
    value: function setBodyHeight(offset) {
      offset = offset || this.state.headerHeight || 0;

      var bodyNode = (0, _reactDom.findDOMNode)(this.refs.body);
      var bodyHeight = void 0;
      if (bodyNode) {
        bodyHeight = bodyNode.offsetHeight - offset;
      } else {
        bodyHeight = 0;
      }

      this.setState({
        bodyHeight: bodyHeight
      });
    }
  }, {
    key: 'setMaxScrollTop',
    value: function setMaxScrollTop(contentHeight, props) {
      props = props || this.props;
      var maxScrollTop = (contentHeight || this.props.contentHeight) - this.state.bodyHeight;

      this.setState({
        maxScrollTop: maxScrollTop
      });

      return maxScrollTop;
    }

    /**
     * Sets isScrolling to false if there is no onScroll
     * registered for 150ms
     */

  }, {
    key: 'toggleIsScrolling',
    value: function toggleIsScrolling() {
      var _this6 = this;

      if (this.disableIsScrollingTimeoutId) {
        clearTimeout(this.disableIsScrollingTimeoutId);
      }
      this.disableIsScrollingTimeoutId = setTimeout(function () {
        delete _this6.disableIsScrollingTimeoutId;
        var newState = {
          isScrolling: false
        };

        if (_this6.props.rowPlaceholder) {
          newState.isPlaceholderActive = false;
        }

        _this6.setState(newState);
      }, 150);
    }
  }, {
    key: 'scrollToIndex',
    value: function scrollToIndex(index) {
      var _ref = arguments.length <= 1 || arguments[1] === undefined ? { position: 'top' } : arguments[1];

      var position = _ref.position;

      // determine height at witch scrolltop should be
      var _p = this.p;
      var bodyHeight = _p.bodyHeight;
      var extraRows = _p.extraRows;
      var rowHeight = _p.rowHeight;


      var scrollTop = void 0;

      if (position === 'top') {
        scrollTop = index * rowHeight;
      } else if (position === 'bottom') {
        scrollTop = index * rowHeight - bodyHeight + rowHeight;
      } else {
        console.error('ScrollToIndex can have position top or bottom.');
        return false;
      }

      this.scrollAt(scrollTop);

      return index;
    }
  }, {
    key: 'scrollAt',
    value: function scrollAt(scrollTop) {
      var _this7 = this;

      (0, _raf2.default)(function () {
        _this7.setState({
          scrollTop: scrollTop
        });
      });

      // trigger scrollbottom
      if (this.p.contentHeight - (this.p.scrollbarWidth + 5) <= scrollTop + this.p.bodyHeight) {
        this.p.onScrollBottom();
      }

      return scrollTop;
    }
  }, {
    key: 'scrollToId',
    value: function scrollToId(id, config) {
      // find index of id
      var index = (0, _getIndexBy2.default)(this.props.data, this.props.idProperty, id);

      return this.scrollToIndex(index, config);
    }

    // set columns depending
    // - there are ColumnGrups with jsx
    // - Columgroups have a prop columns
    // - Columgroups have children

  }, {
    key: 'getNewColumns',
    value: function getNewColumns(props) {
      var _this8 = this;

      props = props || this.props;
      var columnGroups = props.children;
      var columns = void 0;

      // we have children (ColumnGroups)
      if (columnGroups) {
        (function () {
          var startIndex = 0;
          columns = _react2.default.Children.toArray(columnGroups).map(function (columnGroup) {
            var normalizedColumns = _this8.normalizeColumns(columnGroup.props, startIndex);
            startIndex += normalizedColumns.length;
            return normalizedColumns;
          });
        })();
      } else {
        columns = this.normalizeColumns({ columns: props.columns });
      }

      return columns;
    }
  }, {
    key: 'normalizeColumns',
    value: function normalizeColumns(_ref2) {
      var children = _ref2.children;
      var columns = _ref2.columns;
      var startIndex = arguments.length <= 1 || arguments[1] === undefined ? 0 : arguments[1];

      // We want to allow users to use columns configuration as jsx
      // or as an array of config objects
      var normalizedColumns = void 0;

      if (children) {
        // if we have children, we want to take only valid children
        normalizedColumns = _react2.default.Children.toArray(children).filter(function (child) {
          return child && child.props && child.props.isColumn;
        });
      } else {
        // used to add default props
        normalizedColumns = columns.map(function (column) {
          return _react2.default.createElement(_Column2.default, column);
        });
      }

      return normalizedColumns.map(function (c, index) {
        return (0, _objectAssign2.default)({}, c.props, { index: index + startIndex });
      });
    }
  }, {
    key: 'prepareProps',
    value: function prepareProps(props) {
      var isScrollControlled = props.scrollTop != null;
      var scrollTop = isScrollControlled ? props.scrollTop : this.state.scrollTop;

      // buffer is half of extrarows height
      var buffer = props.extraRows / 2 * props.rowHeight;
      var columns = this.state.columns;
      var headerHeight = this.state.headerHeight || 0;

      var hasScroll = this.state.bodyHeight < props.contentHeight;

      return (0, _objectAssign2.default)({}, props, {
        scrollTop: scrollTop,
        buffer: buffer,
        isScrollControlled: isScrollControlled,
        columns: columns,
        headerHeight: headerHeight,
        hasScroll: hasScroll,
        bodyHeight: this.state.bodyHeight,
        maxScrollTop: this.state.maxScrollTop
      });
    }

    // exposed methods

  }, {
    key: 'getScrollTop',
    value: function getScrollTop() {
      return this.state.scrollTop;
    }
  }, {
    key: 'getBodyHeight',
    value: function getBodyHeight() {
      return this.state.bodyHeight;
    }
  }, {
    key: 'getAllColumns',
    value: function getAllColumns() {
      return this.state.columns;
    }
  }, {
    key: 'getFlattenColumns',
    value: function getFlattenColumns() {
      return this.state.flatColumns;
    }
  }]);

  return Body;
}(_reactClass2.default);

Body.defaultProps = {
  rowHeight: 40,
  extraRows: 4,
  defaultScrollTop: 0,
  onRowMouseEnter: function onRowMouseEnter() {},
  onRowMouseLeave: function onRowMouseLeave() {},
  onScrollBottom: function onScrollBottom() {},
  onColumnGroupScroll: function onColumnGroupScroll() {},
  zebraRows: true,
  rowPlaceholderDelay: 300
};

Body.propTypes = {
  loading: _react.PropTypes.bool,
  onScroll: _react.PropTypes.func,
  onRowMouseEnter: _react.PropTypes.func,
  onRowMouseLeave: _react.PropTypes.func,
  onScrollBottom: _react.PropTypes.func,
  onColumnGroupScroll: _react.PropTypes.func,
  zebraRows: _react.PropTypes.bool
};

exports.default = (0, _reactNotifyResize2.default)(Body);